print("import: 'regions'")
import regions

print("import: 'regions.core'")
import regions.core

print("import: 'regions.io'")
import regions.io

print("import: 'regions.shapes'")
import regions.shapes

